////////////////////////////////////////////////////////////////////////////////
//
//  TYPHOON FRAMEWORK
//  Copyright 2013, Typhoon Framework Contributors
//  All Rights Reserved.
//
//  NOTICE: The authors permit you to use, modify, and distribute this file
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

#import "TyphoonDefinition.h"

@class TyphoonConfigPostProcessor;
@protocol TyphoonResource;
@class TyphoonRuntimeArguments;

/**
Declares short-hand definition factory methods for infrastructure components.
*/
@interface TyphoonDefinition (Infrastructure)

@property (nonatomic, strong) TyphoonRuntimeArguments *currentRuntimeArguments;

/** This flag used to distinguish definitions from reference to them. First time, when definition created, processed flag set to NO,
*  but next time, when this definition returned by reference (shortcut with another runtime args) processed flag will be set to YES */
@property (nonatomic) BOOL processed;

/**
* The key of the component. A key is useful when multiple configuration of the same class or protocol are desired - for example
* MasterCardPaymentClient and VisaPaymentClient.
*
* If using the TyphoonBlockComponentFactory style of assembly, the key is automatically generated based on the selector name of the
* component, thus avoiding "magic strings" and providing better integration with IDE refactoring tools.
*/
@property (nonatomic, strong) NSString *key;

/**
* Describes the initializer, ie the selector and arguments that will be used to instantiate this component.
*
* An initializer can be an instance method, a class method, or even a reference to another component's method (see factory property).
*
* If no explicit initializer has been set, returns a default initializer representing the init method.
*
* @see factory
*/
@property (nonatomic, strong, readonly) TyphoonMethod *initializer;

/**
* Returns true if this is a default initializer generated by Typhoon. A manually specified initializer will return false, even if the
* selector is @selector(init)
*/
@property (nonatomic, getter = isInitializerGenerated) BOOL initializerGenerated;

/**
* Returns a definition with the given class and key. In the block-style assembly, keys are auto-generated, however infrastructure components
* may specify their own key.
*/
+ (instancetype)withClass:(Class)clazz key:(NSString *)key;

/**
Factory method for a TyphoonConfigPostProcessor. Don't use it in test targets!
@param fileName The config filename to load. File should be placed in main bundle
@return a definition.
*/
+ (instancetype)configDefinitionWithName:(NSString *)fileName;

/**
 Factory method for a TyphoonConfigPostProcessor.
 @param fileName    The config filename to load.
 @param fileBundle  The bundle, where the config file is placed
 @return a definition.
 */
+ (instancetype)configDefinitionWithName:(NSString *)fileName bundle:(NSBundle *)fileBundle;

/**
Factory method for a TyphoonConfigPostProcessor.
@param filePath The path to config file to load.
@return a definition.
*/
+ (instancetype)configDefinitionWithPath:(NSString *)filePath;

- (BOOL)isCandidateForInjectedClass:(Class)clazz includeSubclasses:(BOOL)includeSubclasses;

- (BOOL)isCandidateForInjectedProtocol:(Protocol *)aProtocol;


@end
